@@ -1,7 +1,9 @@
-use crate::{arange::arange_lerp, gridspace::GridSpace, linspace::Linear};
-use array_init::array_init;
+use crate::{
+    arange::ArangeImpl,
+    gridspace::{GridSpace, GridSpaceInterpolation},
+};
+use array_iter_tools::{ArrayIterator, IntoArrayIterator};
 use core::ops::Range;
-use num_traits::real::Real;
 
 /// Iterator returned by `arange_grid`
 pub type ArangeGrid<T, const N: usize> = GridSpace<T, N>;
@@ -47,49 +49,45 @@ pub trait IntoArangeGrid<F, S, const N: usize> {
     fn into_arange_grid(self, step: S) -> ArangeGrid<F, N>;
 }
 
-impl<F: Real + Linear, const N: usize> IntoArangeGrid<F, [F; N], N> for Range<[F; N]> {
+impl<F: Copy, const N: usize> IntoArangeGrid<F, [F; N], N> for Range<[F; N]>
+where
+    (Range<F>, F): Into<ArangeImpl<F>>,
+{
     fn into_arange_grid(self, step: [F; N]) -> ArangeGrid<F, N> {
         let Self { start, end } = self;
 
-        let mut steps = [0; N];
-        let utils = array_init(|i| {
-            let (util, s) = arange_lerp(start[i]..end[i], step[i]);
-            steps[i] = s;
-            util
-        });
+        let (lerps, steps) = start.into_array_iter().zip(end).zip(step).map(|((start, end), step)| {
+            let ArangeImpl {
+                interpolate,
+                steps,
+            } = (start..end, step).into();
+            (interpolate, steps)
+        }).unzip();
 
-        let mut y = [0; N];
-        y[0] = steps[0];
+        let y = steps.iter().product();
 
-        ArangeGrid {
-            lerps: utils,
-            steps,
-            x: [0; N],
-            y,
-        }
+        ArangeGrid::new(y, GridSpaceInterpolation(lerps, steps))
     }
 }
 
-impl<F: Real + Linear, const N: usize> IntoArangeGrid<F, F, N> for Range<[F; N]> {
+impl<F: Copy, const N: usize> IntoArangeGrid<F, F, N> for Range<[F; N]>
+where
+    (Range<F>, F): Into<ArangeImpl<F>>,
+{
     fn into_arange_grid(self, step: F) -> ArangeGrid<F, N> {
         let Self { start, end } = self;
 
-        let mut steps = [0; N];
-        let utils = array_init(|i| {
-            let (util, s) = arange_lerp(start[i]..end[i], step);
-            steps[i] = s;
-            util
-        });
+        let (lerps, steps) = start.into_array_iter().zip(end).map(|(start, end)| {
+            let ArangeImpl {
+                interpolate,
+                steps,
+            } = (start..end, step).into();
+            (interpolate, steps)
+        }).unzip();
 
-        let mut y = [0; N];
-        y[0] = steps[0];
+        let y = steps.iter().product();
 
-        ArangeGrid {
-            lerps: utils,
-            steps,
-            x: [0; N],
-            y,
-        }
+        ArangeGrid::new(y, GridSpaceInterpolation(lerps, steps))
     }
 }
 
